@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,36 +9,36 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "doxygengenerator.h"
 
+#include <texteditor/convenience.h>
+
 #include <cplusplus/BackwardsScanner.h>
 #include <cplusplus/CppDocument.h>
 
+#include <utils/fileutils.h>
 #include <utils/qtcassert.h>
 
 #include <QStringBuilder>
 #include <QTextDocument>
 #include <QDebug>
 
+#include <limits>
+
 using namespace CppTools;
 using namespace CPlusPlus;
 
@@ -69,8 +69,24 @@ void DoxygenGenerator::setAddLeadingAsterisks(bool add)
     m_addLeadingAsterisks = add;
 }
 
-QString DoxygenGenerator::generate(QTextCursor cursor)
+static int lineBeforeCursor(const QTextCursor &cursor)
 {
+    int line, column;
+    const bool converted = TextEditor::Convenience::convertPosition(cursor.document(),
+                                                                    cursor.position(),
+                                                                    &line,
+                                                                    &column);
+    QTC_ASSERT(converted, return std::numeric_limits<int>::max());
+
+    return line - 1;
+}
+
+QString DoxygenGenerator::generate(QTextCursor cursor,
+                                   const CPlusPlus::Snapshot &snapshot,
+                                   const Utils::FileName &documentFilePath)
+{
+    const QTextCursor initialCursor = cursor;
+
     const QChar &c = cursor.document()->characterAt(cursor.position());
     if (!c.isLetter() && c != QLatin1Char('_'))
         return QString();
@@ -80,7 +96,7 @@ QString DoxygenGenerator::generate(QTextCursor cursor)
     QTextBlock block = cursor.block();
     while (block.isValid()) {
         const QString &text = block.text();
-        const QList<Token> &tks = lexer(text);
+        const Tokens &tks = lexer(text);
         foreach (const Token &tk, tks) {
             if (tk.is(T_SEMICOLON) || tk.is(T_LBRACE)) {
                 // No need to continue beyond this, we might already have something meaningful.
@@ -105,8 +121,9 @@ QString DoxygenGenerator::generate(QTextCursor cursor)
     if (declCandidate.endsWith(QLatin1Char('{')))
         declCandidate.append(QLatin1Char('}'));
 
-    Document::Ptr doc = Document::create(QLatin1String("<doxygen>"));
-    doc->setUtf8Source(declCandidate.toUtf8());
+    Document::Ptr doc = snapshot.preprocessedDocument(declCandidate.toUtf8(),
+                                                      documentFilePath,
+                                                      lineBeforeCursor(initialCursor));
     doc->parse(Document::ParseDeclaration);
     doc->check(Document::FastCheck);
 
@@ -119,7 +136,7 @@ QString DoxygenGenerator::generate(QTextCursor cursor)
     return generate(cursor, doc->translationUnit()->ast()->asDeclaration());
 }
 
-QString DoxygenGenerator::generate(QTextCursor cursor, CPlusPlus::DeclarationAST *decl)
+QString DoxygenGenerator::generate(QTextCursor cursor, DeclarationAST *decl)
 {
     SpecifierAST *spec = 0;
     DeclaratorAST *decltr = 0;
