@@ -27,10 +27,12 @@
 
 #include "debugger_global.h"
 #include "debuggerconstants.h"
+#include "debuggeritem.h"
 #include "debuggerprotocol.h"
 #include "debuggerstartparameters.h"
 
 #include <projectexplorer/devicesupport/idevice.h>
+#include <projectexplorer/runnables.h>
 #include <texteditor/textmark.h>
 
 #include <QObject>
@@ -89,15 +91,13 @@ public:
     bool breakOnMain = false;
     bool multiProcess = false; // Whether to set detach-on-fork off.
 
-    QString debuggerCommand;
-    QString coreFile;
+    ProjectExplorer::StandardRunnable debugger;
     QString overrideStartScript; // Used in attach to core and remote debugging
     QString startMessage; // First status message shown.
     QString debugInfoLocation; // Gdb "set-debug-file-directory".
     QStringList debugSourceLocation; // Gdb "directory"
     QString serverStartScript;
     ProjectExplorer::IDevice::ConstPtr device;
-    QString sysRoot;
     bool isSnapshot = false; // Set if created internally.
     ProjectExplorer::Abi toolChainAbi;
 
@@ -122,7 +122,8 @@ public:
 class UpdateParameters
 {
 public:
-    UpdateParameters() {}
+    UpdateParameters(const QString &partialVariable = QString()) :
+        partialVariable(partialVariable) {}
 
     QStringList partialVariables() const
     {
@@ -138,11 +139,11 @@ public:
 class Location
 {
 public:
-    Location() { init(); }
-    Location(quint64 address) { init(); m_address = address; }
-    Location(const QString &file) { init(); m_fileName = file; }
+    Location() {}
+    Location(quint64 address) { m_address = address; }
+    Location(const QString &file) { m_fileName = file; }
     Location(const QString &file, int line, bool marker = true)
-        { init(); m_lineNumber = line; m_fileName = file; m_needsMarker = marker; }
+        { m_lineNumber = line; m_fileName = file; m_needsMarker = marker; }
     Location(const StackFrame &frame, bool marker = true);
     QString fileName() const { return m_fileName; }
     QString functionName() const { return m_functionName; }
@@ -160,16 +161,14 @@ public:
     quint64 address() const { return m_address; }
 
 private:
-    void init() { m_needsMarker = false; m_needsRaise = true; m_lineNumber = -1;
-        m_address = 0; m_hasDebugInfo = true; }
-    bool m_needsMarker;
-    bool m_needsRaise;
-    bool m_hasDebugInfo;
-    int m_lineNumber;
+    bool m_needsMarker = false;
+    bool m_needsRaise = true;
+    bool m_hasDebugInfo = true;
+    int m_lineNumber = -1;
     QString m_fileName;
     QString m_functionName;
     QString m_from;
-    quint64 m_address;
+    quint64 m_address = 0;
 };
 
 enum LocationType { UnknownLocation, LocationByFile, LocationByAddress };
@@ -207,20 +206,10 @@ public:
     virtual void prepareForRestart() {}
 
     virtual void watchPoint(const QPoint &);
-
-    enum MemoryViewFlags
-    {
-        MemoryReadOnly = 0x1,      //!< Read-only.
-        MemoryTrackRegister = 0x2, //!< Address parameter is register number to track
-        MemoryView = 0x4           //!< Open a separate view (using the pos-parameter).
-    };
-
     virtual void runCommand(const DebuggerCommand &cmd);
     virtual void openMemoryView(const MemoryViewSetupData &data);
-    virtual void fetchMemory(Internal::MemoryAgent *, QObject *,
-                             quint64 addr, quint64 length);
-    virtual void changeMemory(Internal::MemoryAgent *, QObject *,
-                              quint64 addr, const QByteArray &data);
+    virtual void fetchMemory(MemoryAgent *, quint64 addr, quint64 length);
+    virtual void changeMemory(MemoryAgent *, quint64 addr, const QByteArray &data);
     virtual void updateMemoryViews();
     virtual void openDisassemblerView(const Internal::Location &location);
     virtual void fetchDisassembler(Internal::DisassemblerAgent *);
@@ -328,8 +317,6 @@ public:
 
 signals:
     void stateChanged(Debugger::DebuggerState state);
-    // A new stack frame is on display including locals.
-    void stackFrameCompleted();
     /*
      * For "external" clients of a debugger run control that needs to do
      * further setup before the debugger is started (e.g. RemoteLinux).
@@ -428,7 +415,7 @@ protected:
     static QString msgStoppedByException(const QString &description,
         const QString &threadId);
     static QString msgInterrupted();
-    void showStoppedBySignalMessageBox(const QString meaning, QString name);
+    bool showStoppedBySignalMessageBox(const QString meaning, QString name);
     void showStoppedByExceptionMessageBox(const QString &description);
 
     bool isStateDebugging() const;
