@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -73,7 +68,7 @@ namespace Internal {
 
 class SelectRemoteFileDialog : public QDialog
 {
-    Q_OBJECT
+    Q_DECLARE_TR_FUNCTIONS(Debugger::Internal::SelectRemoteFileDialog)
 
 public:
     explicit SelectRemoteFileDialog(QWidget *parent);
@@ -82,14 +77,13 @@ public:
     QString localFile() const { return m_localFile; }
     QString remoteFile() const { return m_remoteFile; }
 
-private slots:
-    void handleSftpOperationFinished(QSsh::SftpJobId, const QString &error);
+private:
+    void handleSftpOperationFinished(SftpJobId, const QString &error);
     void handleSftpOperationFailed(const QString &errorMessage);
     void handleConnectionError(const QString &errorMessage);
     void handleRemoteError(const QString &errorMessage);
     void selectFile();
 
-private:
     QSortFilterProxyModel m_model;
     SftpFileSystemModel m_fileSystemModel;
     QTreeView *m_fileSystemView;
@@ -128,13 +122,15 @@ SelectRemoteFileDialog::SelectRemoteFileDialog(QWidget *parent)
     layout->addWidget(m_textBrowser);
     layout->addWidget(m_buttonBox);
 
-    QObject::connect(m_buttonBox, SIGNAL(rejected()), SLOT(reject()));
-    QObject::connect(m_buttonBox, SIGNAL(accepted()), SLOT(selectFile()));
+    connect(m_buttonBox, &QDialogButtonBox::rejected,
+            this, &QDialog::reject);
+    connect(m_buttonBox, &QDialogButtonBox::accepted,
+            this, &SelectRemoteFileDialog::selectFile);
 
-    connect(&m_fileSystemModel, SIGNAL(sftpOperationFailed(QString)),
-            SLOT(handleSftpOperationFailed(QString)));
-    connect(&m_fileSystemModel, SIGNAL(connectionError(QString)),
-            SLOT(handleConnectionError(QString)));
+    connect(&m_fileSystemModel, &SftpFileSystemModel::sftpOperationFailed,
+            this, &SelectRemoteFileDialog::handleSftpOperationFailed);
+    connect(&m_fileSystemModel, &SftpFileSystemModel::connectionError,
+            this, &SelectRemoteFileDialog::handleConnectionError);
 }
 
 void SelectRemoteFileDialog::attachToDevice(Kit *k)
@@ -143,7 +139,7 @@ void SelectRemoteFileDialog::attachToDevice(Kit *k)
     QTC_ASSERT(k, return);
     IDevice::ConstPtr device = DeviceKitInformation::device(k);
     QTC_ASSERT(device, return);
-    QSsh::SshConnectionParameters sshParams = device->sshParameters();
+    SshConnectionParameters sshParams = device->sshParameters();
     m_fileSystemModel.setSshConnection(sshParams);
 }
 
@@ -159,7 +155,7 @@ void SelectRemoteFileDialog::handleConnectionError(const QString &errorMessage)
     //reject();
 }
 
-void SelectRemoteFileDialog::handleSftpOperationFinished(QSsh::SftpJobId, const QString &error)
+void SelectRemoteFileDialog::handleSftpOperationFinished(SftpJobId, const QString &error)
 {
     if (error.isEmpty()) {
         m_textBrowser->append(tr("Download of remote file succeeded."));
@@ -184,8 +180,8 @@ void SelectRemoteFileDialog::selectFile()
     m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
     m_fileSystemView->setEnabled(false);
 
-    connect(&m_fileSystemModel, SIGNAL(sftpOperationFinished(QSsh::SftpJobId,QString)),
-            SLOT(handleSftpOperationFinished(QSsh::SftpJobId,QString)));
+    connect(&m_fileSystemModel, &SftpFileSystemModel::sftpOperationFinished,
+            this, &SelectRemoteFileDialog::handleSftpOperationFinished);
 
     {
         QTemporaryFile localFile(QDir::tempPath() + QLatin1String("/remotecore-XXXXXX"));
@@ -262,7 +258,7 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     d->buttonBox->button(QDialogButtonBox::Ok)->setDefault(true);
     d->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
 
-    d->kitChooser = new DebuggerKitChooser(DebuggerKitChooser::RemoteDebugging, this);
+    d->kitChooser = new DebuggerKitChooser(DebuggerKitChooser::AnyDebugging, this);
     d->kitChooser->populate();
 
     d->forceLocalCheckBox = new QCheckBox(this);
@@ -271,7 +267,7 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     d->forceLocalLabel->setBuddy(d->forceLocalCheckBox);
 
     d->remoteCoreFileName = new QLineEdit(this);
-    d->selectRemoteCoreButton = new QPushButton(tr("Browse..."), this);
+    d->selectRemoteCoreButton = new QPushButton(PathChooser::browseButtonLabel(), this);
 
     d->localCoreFileName = new PathChooser(this);
     d->localCoreFileName->setHistoryCompleter(QLatin1String("Debugger.CoreFile.History"));
@@ -288,12 +284,12 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     d->overrideStartScriptFileName->setExpectedKind(PathChooser::File);
     d->overrideStartScriptFileName->setPromptDialogTitle(tr("Select Startup Script"));
 
-    QHBoxLayout *coreLayout = new QHBoxLayout;
+    auto coreLayout = new QHBoxLayout;
     coreLayout->addWidget(d->localCoreFileName);
     coreLayout->addWidget(d->remoteCoreFileName);
     coreLayout->addWidget(d->selectRemoteCoreButton);
 
-    QFormLayout *formLayout = new QFormLayout;
+    auto formLayout = new QFormLayout;
     formLayout->setContentsMargins(0, 0, 0, 0);
     formLayout->setHorizontalSpacing(6);
     formLayout->setVerticalSpacing(6);
@@ -303,13 +299,13 @@ AttachCoreDialog::AttachCoreDialog(QWidget *parent)
     formLayout->addRow(tr("&Executable:"), d->localExecFileName);
     formLayout->addRow(tr("Override &start script:"), d->overrideStartScriptFileName);
 
-    QFrame *line = new QFrame(this);
+    auto line = new QFrame(this);
     line->setFrameShape(QFrame::HLine);
     line->setFrameShadow(QFrame::Sunken);
 
     formLayout->addRow(d->buttonBox);
 
-    QVBoxLayout *vboxLayout = new QVBoxLayout(this);
+    auto vboxLayout = new QVBoxLayout(this);
     vboxLayout->addLayout(formLayout);
     vboxLayout->addStretch();
     vboxLayout->addWidget(line);
@@ -323,14 +319,14 @@ AttachCoreDialog::~AttachCoreDialog()
 
 int AttachCoreDialog::exec()
 {
-    connect(d->selectRemoteCoreButton, SIGNAL(clicked()), SLOT(selectRemoteCoreFile()));
-    connect(d->remoteCoreFileName, SIGNAL(textChanged(QString)), SLOT(coreFileChanged(QString)));
-    connect(d->localExecFileName, SIGNAL(changed(QString)), SLOT(changed()));
-    connect(d->localCoreFileName, SIGNAL(changed(QString)), SLOT(coreFileChanged(QString)));
-    connect(d->forceLocalCheckBox, SIGNAL(stateChanged(int)), SLOT(changed()));
-    connect(d->kitChooser, SIGNAL(currentIndexChanged(int)), SLOT(changed()));
-    connect(d->buttonBox, SIGNAL(rejected()), SLOT(reject()));
-    connect(d->buttonBox, SIGNAL(accepted()), SLOT(accept()));
+    connect(d->selectRemoteCoreButton, &QAbstractButton::clicked, this, &AttachCoreDialog::selectRemoteCoreFile);
+    connect(d->remoteCoreFileName, &QLineEdit::textChanged, this, &AttachCoreDialog::coreFileChanged);
+    connect(d->localExecFileName, &PathChooser::rawPathChanged, this, &AttachCoreDialog::changed);
+    connect(d->localCoreFileName, &PathChooser::rawPathChanged, this, &AttachCoreDialog::coreFileChanged);
+    connect(d->forceLocalCheckBox, &QCheckBox::stateChanged, this, &AttachCoreDialog::changed);
+    connect(d->kitChooser, &KitChooser::currentIndexChanged, this, &AttachCoreDialog::changed);
+    connect(d->buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
+    connect(d->buttonBox, &QDialogButtonBox::accepted, this, &QDialog::accept);
     changed();
 
     AttachCoreDialogPrivate::State st = d->getDialogState(*this);
@@ -364,14 +360,15 @@ bool AttachCoreDialog::useLocalCoreFile() const
 
 void AttachCoreDialog::coreFileChanged(const QString &core)
 {
-    if (!Utils::HostOsInfo::isWindowsHost() && QFile::exists(core)) {
+    if (!HostOsInfo::isWindowsHost() && QFile::exists(core)) {
         Kit *k = d->kitChooser->currentKit();
         QTC_ASSERT(k, return);
-        FileName cmd = DebuggerKitInformation::debuggerCommand(k);
-        bool isCore = false;
-        const QString exe = readExecutableNameFromCore(cmd.toString(), core, &isCore);
-        if (!exe.isEmpty())
-            d->localExecFileName->setFileName(FileName::fromString(exe));
+        StandardRunnable debugger = DebuggerKitInformation::runnable(k);
+        GdbCoreEngine::CoreInfo cinfo = GdbCoreEngine::readExecutableNameFromCore(debugger, core);
+        if (!cinfo.foundExecutableName.isEmpty())
+            d->localExecFileName->setFileName(FileName::fromString(cinfo.foundExecutableName));
+        else if (!d->localExecFileName->isValid() && !cinfo.rawStringFromCore.isEmpty())
+            d->localExecFileName->setFileName(FileName::fromString(cinfo.rawStringFromCore));
     }
     changed();
 }
@@ -439,7 +436,7 @@ QString AttachCoreDialog::remoteCoreFile() const
     return d->remoteCoreFileName->text();
 }
 
-void AttachCoreDialog::setKitId(Core::Id id)
+void AttachCoreDialog::setKitId(Id id)
 {
     d->kitChooser->setCurrentKitId(id);
 }
@@ -471,5 +468,3 @@ void AttachCoreDialog::setOverrideStartScript(const QString &scriptName)
 
 } // namespace Internal
 } // namespace Debugger
-
-#include "loadcoredialog.moc"
