@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,6 +28,7 @@
 #include "bookmarkmanager.h"
 #include "contentwindow.h"
 #include "helpconstants.h"
+#include "helpicons.h"
 #include "helpplugin.h"
 #include "helpviewer.h"
 #include "indexwindow.h"
@@ -47,13 +43,14 @@
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/findplaceholder.h>
-#include <coreplugin/helpmanager.h>
 #include <coreplugin/minisplitter.h>
 #include <coreplugin/sidebar.h>
 #include <texteditor/texteditorconstants.h>
 #include <utils/qtcassert.h>
 #include <utils/styledbar.h>
+#include <utils/utilsicons.h>
 
+#include <QCoreApplication>
 #include <QHBoxLayout>
 #include <QHelpEngine>
 #include <QHelpSearchEngine>
@@ -70,23 +67,22 @@ static const char kModeSideBarSettingsKey[] = "Help/ModeSideBar";
 namespace Help {
 namespace Internal {
 
+static void openUrlInWindow(const QUrl &url)
+{
+    HelpViewer *viewer = HelpPlugin::viewerForHelpViewerLocation(Core::HelpManager::ExternalHelpAlways);
+    if (QTC_GUARD(viewer))
+        viewer->setSource(url);
+    Core::ICore::raiseWindow(viewer);
+}
+
+static bool isBookmarkable(const QUrl &url)
+{
+    return !url.isEmpty() && url != QUrl(Help::Constants::AboutBlank);
+}
+
 HelpWidget::HelpWidget(const Core::Context &context, WidgetStyle style, QWidget *parent) :
     QWidget(parent),
-    m_style(style),
-    m_toggleSideBarAction(0),
-    m_switchToHelp(0),
-    m_filterComboBox(0),
-    m_closeAction(0),
-    m_scaleUp(0),
-    m_scaleDown(0),
-    m_resetScale(0),
-    m_printer(0),
-    m_sideBar(0),
-    m_contentsAction(0),
-    m_indexAction(0),
-    m_bookmarkAction(0),
-    m_searchAction(0),
-    m_openPagesAction(0)
+    m_style(style)
 {
     m_viewerStack = new QStackedWidget;
 
@@ -131,16 +127,19 @@ HelpWidget::HelpWidget(const Core::Context &context, WidgetStyle style, QWidget
         setAttribute(Qt::WA_QuitOnClose, false); // don't prevent Qt Creator from closing
     }
     if (style != SideBarWidget) {
-        m_toggleSideBarAction = new QAction(QIcon(QLatin1String(Core::Constants::ICON_TOGGLE_SIDEBAR)),
-                                            tr(Core::Constants::TR_SHOW_SIDEBAR), toolBar);
+        m_toggleSideBarAction = new QAction(Utils::Icons::TOGGLE_SIDEBAR_TOOLBAR.icon(),
+                                            QCoreApplication::translate("Core", Core::Constants::TR_SHOW_SIDEBAR),
+                                            toolBar);
         m_toggleSideBarAction->setCheckable(true);
         m_toggleSideBarAction->setChecked(false);
         cmd = Core::ActionManager::registerAction(m_toggleSideBarAction,
                                                   Core::Constants::TOGGLE_SIDEBAR, context);
         connect(m_toggleSideBarAction, &QAction::toggled, m_toggleSideBarAction,
                 [this](bool checked) {
-                    m_toggleSideBarAction->setText(checked ? tr(Core::Constants::TR_HIDE_SIDEBAR)
-                                                           : tr(Core::Constants::TR_SHOW_SIDEBAR));
+                    m_toggleSideBarAction->setText(
+                        QCoreApplication::translate("Core",
+                                                    checked ? Core::Constants::TR_HIDE_SIDEBAR
+                                                            : Core::Constants::TR_SHOW_SIDEBAR));
                 });
         addSideBar();
         m_toggleSideBarAction->setChecked(m_sideBar->isVisibleTo(this));
@@ -153,23 +152,21 @@ HelpWidget::HelpWidget(const Core::Context &context, WidgetStyle style, QWidget
         layout->addWidget(Core::Command::toolButtonWithAppendedShortcut(m_toggleSideBarAction, cmd));
 
     if (style != ModeWidget) {
-        m_switchToHelp = new QAction(tr("Go to Help Mode"), toolBar);
+        m_switchToHelp = new QAction(tr("Open in Help Mode"), toolBar);
         cmd = Core::ActionManager::registerAction(m_switchToHelp, Constants::CONTEXT_HELP, context);
-        connect(m_switchToHelp, SIGNAL(triggered()), this, SLOT(helpModeButtonClicked()));
+        connect(m_switchToHelp, &QAction::triggered, this, &HelpWidget::helpModeButtonClicked);
         layout->addWidget(Core::Command::toolButtonWithAppendedShortcut(m_switchToHelp, cmd));
     }
 
-    m_homeAction = new QAction(QIcon(QLatin1String(":/help/images/home.png")),
-        tr("Home"), this);
+    m_homeAction = new QAction(Icons::HOME_TOOLBAR.icon(), tr("Home"), this);
     cmd = Core::ActionManager::registerAction(m_homeAction, Constants::HELP_HOME, context);
     connect(m_homeAction, &QAction::triggered, this, &HelpWidget::goHome);
     layout->addWidget(Core::Command::toolButtonWithAppendedShortcut(m_homeAction, cmd));
 
-    m_backAction = new QAction(QIcon(QLatin1String(":/help/images/previous.png")),
-        tr("Back"), toolBar);
+    m_backAction = new QAction(Utils::Icons::PREV_TOOLBAR.icon(), tr("Back"), toolBar);
     connect(m_backAction, &QAction::triggered, this, &HelpWidget::backward);
     m_backMenu = new QMenu(toolBar);
-    connect(m_backMenu, SIGNAL(aboutToShow()), this, SLOT(updateBackMenu()));
+    connect(m_backMenu, &QMenu::aboutToShow, this, &HelpWidget::updateBackMenu);
     m_backAction->setMenu(m_backMenu);
     cmd = Core::ActionManager::registerAction(m_backAction, Constants::HELP_PREVIOUS, context);
     cmd->setDefaultKeySequence(QKeySequence::Back);
@@ -177,11 +174,10 @@ HelpWidget::HelpWidget(const Core::Context &context, WidgetStyle style, QWidget
     button->setPopupMode(QToolButton::DelayedPopup);
     layout->addWidget(button);
 
-    m_forwardAction = new QAction(QIcon(QLatin1String(":/help/images/next.png")),
-        tr("Forward"), toolBar);
+    m_forwardAction = new QAction(Utils::Icons::NEXT_TOOLBAR.icon(), tr("Forward"), toolBar);
     connect(m_forwardAction, &QAction::triggered, this, &HelpWidget::forward);
     m_forwardMenu = new QMenu(toolBar);
-    connect(m_forwardMenu, SIGNAL(aboutToShow()), this, SLOT(updateForwardMenu()));
+    connect(m_forwardMenu, &QMenu::aboutToShow, this, &HelpWidget::updateForwardMenu);
     m_forwardAction->setMenu(m_forwardMenu);
     cmd = Core::ActionManager::registerAction(m_forwardAction, Constants::HELP_NEXT, context);
     cmd->setDefaultKeySequence(QKeySequence::Forward);
@@ -189,8 +185,7 @@ HelpWidget::HelpWidget(const Core::Context &context, WidgetStyle style, QWidget
     button->setPopupMode(QToolButton::DelayedPopup);
     layout->addWidget(button);
 
-    m_addBookmarkAction = new QAction(QIcon(QLatin1String(":/help/images/bookmark.png")),
-        tr("Add Bookmark"), this);
+    m_addBookmarkAction = new QAction(Utils::Icons::BOOKMARK_TOOLBAR.icon(), tr("Add Bookmark"), this);
     cmd = Core::ActionManager::registerAction(m_addBookmarkAction, Constants::HELP_ADDBOOKMARK, context);
     cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+M") : tr("Ctrl+M")));
     connect(m_addBookmarkAction, &QAction::triggered, this, &HelpWidget::addBookmark);
@@ -226,8 +221,7 @@ HelpWidget::HelpWidget(const Core::Context &context, WidgetStyle style, QWidget
     connect(m_copy, &QAction::triggered, this, &HelpWidget::copy);
 
     Core::ActionContainer *advancedMenu = Core::ActionManager::actionContainer(Core::Constants::M_EDIT_ADVANCED);
-    QTC_CHECK(advancedMenu);
-    if (advancedMenu) {
+    if (QTC_GUARD(advancedMenu)) {
         // reuse TextEditor constants to avoid a second pair of menu actions
         m_scaleUp = new QAction(tr("Increase Font Size"), this);
         cmd = Core::ActionManager::registerAction(m_scaleUp, TextEditor::Constants::INCREASE_FONT_SIZE,
@@ -249,9 +243,35 @@ HelpWidget::HelpWidget(const Core::Context &context, WidgetStyle style, QWidget
     }
 
     if (style != ExternalWindow) {
-        m_closeAction = new QAction(QIcon(QLatin1String(Core::Constants::ICON_BUTTON_CLOSE)),
-            QString(), toolBar);
-        connect(m_closeAction, SIGNAL(triggered()), this, SIGNAL(closeButtonClicked()));
+        auto openButton = new QToolButton;
+        openButton->setIcon(Utils::Icons::SPLIT_HORIZONTAL_TOOLBAR.icon());
+        openButton->setPopupMode(QToolButton::InstantPopup);
+        openButton->setProperty("noArrow", true);
+        layout->addWidget(openButton);
+        QMenu *openMenu = new QMenu(openButton);
+        if (m_switchToHelp)
+            openMenu->addAction(m_switchToHelp);
+        if (style == ModeWidget) {
+            QAction *openPage = openMenu->addAction(tr("Open in New Page"));
+            connect(openPage, &QAction::triggered, this, [this]() {
+                if (HelpViewer *viewer = currentViewer())
+                    OpenPagesManager::instance().createPage(viewer->source());
+            });
+        }
+        QAction *openExternal = openMenu->addAction(tr("Open in Window"));
+        connect(openExternal, &QAction::triggered, this, [this]() {
+            if (HelpViewer *viewer = currentViewer()) {
+                openUrlInWindow(viewer->source());
+                if (m_style == SideBarWidget)
+                    emit closeButtonClicked();
+            }
+        });
+        openButton->setMenu(openMenu);
+
+        const Utils::Icon &icon = style == ModeWidget ? Utils::Icons::CLOSE_TOOLBAR
+                                                      : Utils::Icons::CLOSE_SPLIT_RIGHT;
+        m_closeAction = new QAction(icon.icon(), QString(), toolBar);
+        connect(m_closeAction, &QAction::triggered, this, &HelpWidget::closeButtonClicked);
         button = new QToolButton;
         button->setDefaultAction(m_closeAction);
         layout->addWidget(button);
@@ -301,43 +321,42 @@ void HelpWidget::addSideBar()
     bool supportsNewPages = (m_style == ModeWidget);
 
     auto contentWindow = new ContentWindow;
-    auto contentItem = new Core::SideBarItem(contentWindow, QLatin1String(Constants::HELP_CONTENTS));
+    auto contentItem = new Core::SideBarItem(contentWindow, Constants::HELP_CONTENTS);
     contentWindow->setOpenInNewPageActionVisible(supportsNewPages);
     contentWindow->setWindowTitle(HelpPlugin::tr(Constants::SB_CONTENTS));
     connect(contentWindow, &ContentWindow::linkActivated,
             this, &HelpWidget::open);
-    m_contentsAction = new QAction(tr(Constants::SB_CONTENTS), this);
+    m_contentsAction = new QAction(HelpPlugin::tr(Constants::SB_CONTENTS), this);
     cmd = Core::ActionManager::registerAction(m_contentsAction, Constants::HELP_CONTENTS, m_context->context());
     cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+Shift+C")
                                                                   : tr("Ctrl+Shift+C")));
-    shortcutMap.insert(QLatin1String(Constants::HELP_CONTENTS), cmd);
+    shortcutMap.insert(Constants::HELP_CONTENTS, cmd);
 
     auto indexWindow = new IndexWindow();
-    auto indexItem = new Core::SideBarItem(indexWindow, QLatin1String(Constants::HELP_INDEX));
+    auto indexItem = new Core::SideBarItem(indexWindow, Constants::HELP_INDEX);
     indexWindow->setOpenInNewPageActionVisible(supportsNewPages);
     indexWindow->setWindowTitle(HelpPlugin::tr(Constants::SB_INDEX));
     connect(indexWindow, &IndexWindow::linkActivated,
             this, &HelpWidget::open);
     connect(indexWindow, &IndexWindow::linksActivated,
         this, &HelpWidget::showTopicChooser);
-    m_indexAction = new QAction(tr(Constants::SB_INDEX), this);
+    m_indexAction = new QAction(HelpPlugin::tr(Constants::SB_INDEX), this);
     cmd = Core::ActionManager::registerAction(m_indexAction, Constants::HELP_INDEX, m_context->context());
     cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+I")
                                                                   : tr("Ctrl+Shift+I")));
-    shortcutMap.insert(QLatin1String(Constants::HELP_INDEX), cmd);
+    shortcutMap.insert(Constants::HELP_INDEX, cmd);
 
     auto bookmarkWidget = new BookmarkWidget(&LocalHelpManager::bookmarkManager());
     bookmarkWidget->setWindowTitle(HelpPlugin::tr(Constants::SB_BOOKMARKS));
     bookmarkWidget->setOpenInNewPageActionVisible(supportsNewPages);
-    auto bookmarkItem = new Core::SideBarItem(bookmarkWidget,
-                                              QLatin1String(Constants::HELP_BOOKMARKS));
+    auto bookmarkItem = new Core::SideBarItem(bookmarkWidget, Constants::HELP_BOOKMARKS);
     connect(bookmarkWidget, &BookmarkWidget::linkActivated, this, &HelpWidget::setSource);
     m_bookmarkAction = new QAction(tr("Activate Help Bookmarks View"), this);
     cmd = Core::ActionManager::registerAction(m_bookmarkAction, Constants::HELP_BOOKMARKS,
                                               m_context->context());
     cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Alt+Meta+M")
                                                                   : tr("Ctrl+Shift+B")));
-    shortcutMap.insert(QLatin1String(Constants::HELP_BOOKMARKS), cmd);
+    shortcutMap.insert(Constants::HELP_BOOKMARKS, cmd);
 
     auto searchItem = new SearchSideBarItem;
     connect(searchItem, &SearchSideBarItem::linkActivated, this, &HelpWidget::openFromSearch);
@@ -346,20 +365,19 @@ void HelpWidget::addSideBar()
                                               m_context->context());
     cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+/")
                                                                   : tr("Ctrl+Shift+/")));
-    shortcutMap.insert(QLatin1String(Constants::HELP_SEARCH), cmd);
+    shortcutMap.insert(Constants::HELP_SEARCH, cmd);
 
     Core::SideBarItem *openPagesItem = 0;
     if (m_style == ModeWidget) {
         QWidget *openPagesWidget = OpenPagesManager::instance().openPagesWidget();
         openPagesWidget->setWindowTitle(HelpPlugin::tr(Constants::SB_OPENPAGES));
-        openPagesItem = new Core::SideBarItem(openPagesWidget,
-                                              QLatin1String(Constants::HELP_OPENPAGES));
+        openPagesItem = new Core::SideBarItem(openPagesWidget, Constants::HELP_OPENPAGES);
         m_openPagesAction = new QAction(tr("Activate Open Help Pages View"), this);
         cmd = Core::ActionManager::registerAction(m_openPagesAction, Constants::HELP_OPENPAGES,
                                                   m_context->context());
         cmd->setDefaultKeySequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+O")
                                                                       : tr("Ctrl+Shift+O")));
-        shortcutMap.insert(QLatin1String(Constants::HELP_OPENPAGES), cmd);
+        shortcutMap.insert(Constants::HELP_OPENPAGES, cmd);
     }
 
     QList<Core::SideBarItem *> itemList;
@@ -381,20 +399,20 @@ void HelpWidget::addSideBar()
     m_sideBarSplitter->setSizes(QList<int>() << m_sideBar->size().width() << 300);
 
     connect(m_contentsAction, &QAction::triggered, m_sideBar, [this]() {
-        m_sideBar->activateItem(QLatin1String(Constants::HELP_CONTENTS));
+        m_sideBar->activateItem(Constants::HELP_CONTENTS);
     });
     connect(m_indexAction, &QAction::triggered, m_sideBar, [this]() {
-        m_sideBar->activateItem(QLatin1String(Constants::HELP_INDEX));
+        m_sideBar->activateItem(Constants::HELP_INDEX);
     });
     connect(m_bookmarkAction, &QAction::triggered, m_sideBar, [this]() {
-        m_sideBar->activateItem(QLatin1String(Constants::HELP_BOOKMARKS));
+        m_sideBar->activateItem(Constants::HELP_BOOKMARKS);
     });
     connect(m_searchAction, &QAction::triggered, m_sideBar, [this]() {
-        m_sideBar->activateItem(QLatin1String(Constants::HELP_SEARCH));
+        m_sideBar->activateItem(Constants::HELP_SEARCH);
     });
     if (m_openPagesAction) {
         connect(m_openPagesAction, &QAction::triggered, m_sideBar, [this]() {
-            m_sideBar->activateItem(QLatin1String(Constants::HELP_OPENPAGES));
+            m_sideBar->activateItem(Constants::HELP_OPENPAGES);
         });
     }
 }
@@ -403,9 +421,9 @@ QString HelpWidget::sideBarSettingsKey() const
 {
     switch (m_style) {
     case ModeWidget:
-        return QLatin1String(kModeSideBarSettingsKey);
+        return QString(kModeSideBarSettingsKey);
     case ExternalWindow:
-        return QLatin1String(kWindowSideBarSettingsKey);
+        return QString(kWindowSideBarSettingsKey);
     case SideBarWidget:
         QTC_CHECK(false);
         break;
@@ -423,6 +441,7 @@ void HelpWidget::setCurrentViewer(HelpViewer *viewer)
     m_viewerStack->setCurrentWidget(viewer);
     m_backAction->setEnabled(viewer->isBackwardAvailable());
     m_forwardAction->setEnabled(viewer->isForwardAvailable());
+    m_addBookmarkAction->setEnabled(isBookmarkable(viewer->source()));
     if (m_style == ExternalWindow)
         updateWindowTitle();
     emit sourceChanged(viewer->source());
@@ -437,11 +456,13 @@ void HelpWidget::addViewer(HelpViewer *viewer)
 {
     m_viewerStack->addWidget(viewer);
     viewer->setFocus(Qt::OtherFocusReason);
-    if (m_style == SideBarWidget || m_style == ExternalWindow)
-        viewer->setOpenInNewPageActionVisible(false);
+    viewer->setActionVisible(HelpViewer::Action::NewPage, m_style == ModeWidget);
+    viewer->setActionVisible(HelpViewer::Action::ExternalWindow, m_style != ExternalWindow);
     connect(viewer, &HelpViewer::sourceChanged, this, [viewer, this](const QUrl &url) {
-        if (currentViewer() == viewer)
+        if (currentViewer() == viewer) {
+            m_addBookmarkAction->setEnabled(isBookmarkable(url));
             emit sourceChanged(url);
+        }
     });
     connect(viewer, &HelpViewer::forwardAvailable, this, [viewer, this](bool available) {
         if (currentViewer() == viewer)
@@ -455,9 +476,13 @@ void HelpWidget::addViewer(HelpViewer *viewer)
         print(viewer);
     });
     if (m_style == ExternalWindow)
-        connect(viewer, SIGNAL(titleChanged()), this, SLOT(updateWindowTitle()));
+        connect(viewer, &HelpViewer::titleChanged, this, &HelpWidget::updateWindowTitle);
 
     connect(viewer, &HelpViewer::loadFinished, this, &HelpWidget::highlightSearchTerms);
+    connect(viewer, &HelpViewer::newPageRequested, [](const QUrl &url) {
+        OpenPagesManager::instance().createPage(url);
+    });
+    connect(viewer, &HelpViewer::externalPageRequested, this, &openUrlInWindow);
 
     updateCloseButton();
 }
@@ -474,15 +499,6 @@ void HelpWidget::removeViewerAt(int index)
     updateCloseButton();
 }
 
-void HelpWidget::setViewerFont(const QFont &font)
-{
-    for (int i = 0; i < m_viewerStack->count(); ++i) {
-        HelpViewer *viewer = qobject_cast<HelpViewer *>(m_viewerStack->widget(i));
-        QTC_ASSERT(viewer, continue);
-        viewer->setFont(font);
-    }
-}
-
 int HelpWidget::viewerCount() const
 {
     return m_viewerStack->count();
@@ -576,8 +592,7 @@ void HelpWidget::helpModeButtonClicked()
 void HelpWidget::updateCloseButton()
 {
     if (m_style == ModeWidget) {
-        const bool closeOnReturn = Core::HelpManager::customValue(QLatin1String("ReturnOnClose"),
-            false).toBool();
+        const bool closeOnReturn = LocalHelpManager::returnOnClose();
         m_closeAction->setEnabled(closeOnReturn || m_viewerStack->count() > 1);
     }
 }
@@ -594,7 +609,7 @@ void HelpWidget::addBookmark()
     QTC_ASSERT(viewer, return);
 
     const QString &url = viewer->source().toString();
-    if (url.isEmpty() || url == Help::Constants::AboutBlank)
+    if (!isBookmarkable(url))
         return;
 
     BookmarkManager *manager = &LocalHelpManager::bookmarkManager();
