@@ -72,6 +72,8 @@
 #include <QMessageBox>
 #include <QVariantMap>
 
+#include <algorithm>
+
 using namespace Core;
 using namespace ProjectExplorer;
 using namespace Utils;
@@ -84,14 +86,11 @@ namespace Internal {
 // --------------------------------------------------------------------
 
 static const char CONFIG_CPP_MODULE[] = "cpp";
-static const char CONFIG_CXXFLAGS[] = "cxxFlags";
-static const char CONFIG_CFLAGS[] = "cFlags";
 static const char CONFIG_DEFINES[] = "defines";
 static const char CONFIG_INCLUDEPATHS[] = "includePaths";
 static const char CONFIG_SYSTEM_INCLUDEPATHS[] = "systemIncludePaths";
 static const char CONFIG_FRAMEWORKPATHS[] = "frameworkPaths";
 static const char CONFIG_SYSTEM_FRAMEWORKPATHS[] = "systemFrameworkPaths";
-static const char CONFIG_PRECOMPILEDHEADER[] = "precompiledHeader";
 
 // --------------------------------------------------------------------
 // QbsProject:
@@ -158,29 +157,53 @@ void QbsProject::projectLoaded()
     m_parsingDelay.start(0);
 }
 
-static void collectFilesForProject(const qbs::ProjectData &project, QSet<QString> &result)
+static void collectFilesForProject(const qbs::ProjectData &project, Project::FilesMode mode,
+                                   QSet<QString> &result)
 {
-    result.insert(project.location().filePath());
+    if (mode & Project::SourceFiles)
+        result.insert(project.location().filePath());
+
     foreach (const qbs::ProductData &prd, project.products()) {
-        foreach (const qbs::GroupData &grp, prd.groups()) {
-            foreach (const QString &file, grp.allFilePaths())
-                result.insert(file);
-            result.insert(grp.location().filePath());
+        if (mode & Project::SourceFiles) {
+            foreach (const qbs::GroupData &grp, prd.groups()) {
+                foreach (const QString &file, grp.allFilePaths())
+                    result.insert(file);
+                result.insert(grp.location().filePath());
+            }
+            result.insert(prd.location().filePath());
+        }
+        if (mode & Project::GeneratedFiles) {
+            foreach (const qbs::ProductData &prd, project.products()) {
+                foreach (const qbs::ArtifactData &artifact, prd.generatedArtifacts()) {
+                    // A list of human-readable file types that we can reasonably expect
+                    // to get generated during a build. Extend as needed.
+                    static const QSet<QString> sourceTags = {
+                        QLatin1String("c"), QLatin1String("cpp"), QLatin1String("hpp"),
+                        QLatin1String("objc"), QLatin1String("objcpp"),
+                        QLatin1String("c_pch_src"), QLatin1String("cpp_pch_src"),
+                        QLatin1String("objc_pch_src"), QLatin1String("objcpp_pch_src"),
+                        QLatin1String("asm"), QLatin1String("asm_cpp"),
+                        QLatin1String("linkerscript"),
+                        QLatin1String("qrc"), QLatin1String("java.java")
+                    };
+                    if (artifact.fileTags().toSet().intersects(sourceTags))
+                        result.insert(artifact.filePath());
+                }
+            }
         }
-        result.insert(prd.location().filePath());
     }
+
     foreach (const qbs::ProjectData &subProject, project.subProjects())
-        collectFilesForProject(subProject, result);
+        collectFilesForProject(subProject, mode, result);
 }
 
 QStringList QbsProject::files(Project::FilesMode fileMode) const
 {
-    Q_UNUSED(fileMode);
-    qCDebug(qbsPmLog) << Q_FUNC_INFO << m_qbsProject.isValid() << isParsing();
+    qCDebug(qbsPmLog) << Q_FUNC_INFO << fileMode << m_qbsProject.isValid() << isParsing();
     if (!m_qbsProject.isValid() || isParsing())
         return QStringList();
     QSet<QString> result;
-    collectFilesForProject(m_projectData, result);
+    collectFilesForProject(m_projectData, fileMode, result);
     result.unite(m_qbsProject.buildSystemFiles());
     qCDebug(qbsPmLog) << "file count:" << result.count();
     return result.toList();
@@ -241,7 +264,7 @@ bool QbsProject::ensureWriteableQbsFile(const QString &file)
             bool makeWritable = QFile::setPermissions(file, fi.permissions() | QFile::WriteUser);
             if (!makeWritable) {
                 QMessageBox::warning(ICore::mainWindow(),
-                                     tr("Failed!"),
+                                     tr("Failed"),
                                      tr("Could not write project file %1.").arg(file));
                 return false;
             }
@@ -289,7 +312,7 @@ bool QbsProject::addFilesToProduct(QbsBaseProjectNode *node, const QStringList &
     if (notAdded->count() != filePaths.count()) {
         m_projectData = m_qbsProject.projectData();
         QbsGroupNode::setupFiles(node, reRetrieveGroupData(productData, groupData),
-                                 allPaths, QFileInfo(productFilePath).absolutePath(), true);
+                                 allPaths, QFileInfo(productFilePath).absolutePath(), true, false);
         rootProjectNode()->update();
         emit fileListChanged();
     }
@@ -318,7 +341,7 @@ bool QbsProject::removeFilesFromProduct(QbsBaseProjectNode *node, const QStringL
     if (notRemoved->count() != filePaths.count()) {
         m_projectData = m_qbsProject.projectData();
         QbsGroupNode::setupFiles(node, reRetrieveGroupData(productData, groupData), allPaths,
-                                 QFileInfo(productFilePath).absolutePath(), true);
+                                 QFileInfo(productFilePath).absolutePath(), true, false);
         rootProjectNode()->update();
         emit fileListChanged();
     }
@@ -483,22 +506,6 @@ void QbsProject::handleQbsParsingDone(bool success)
         m_qbsUpdateFutureInterface->reportCanceled();
     }
 
-    bool hasTargetArtifacts = false;
-    if (dataChanged) {
-        qCDebug(qbsPmLog) << "Project data changed.";
-        foreach (const qbs::ProductData &product, m_projectData.allProducts()) {
-            if (!product.targetArtifacts().isEmpty()) {
-                hasTargetArtifacts = true;
-                break;
-            }
-        }
-        if (!hasTargetArtifacts) {
-            qCDebug(qbsPmLog) << "No target artifacts present, executing rules";
-            m_qbsProjectParser->startRuleExecution();
-            return;
-        }
-    }
-
     m_qbsProjectParser->deleteLater();
     m_qbsProjectParser = 0;
     m_qbsUpdateFutureInterface->reportFinished();
@@ -508,6 +515,7 @@ void QbsProject::handleQbsParsingDone(bool success)
     if (dataChanged)
         updateAfterParse();
     emit projectParsingDone(success);
+    emit parsingFinished();
 }
 
 void QbsProject::handleRuleExecutionDone()
@@ -627,7 +635,12 @@ void QbsProject::cancelParsing()
 void QbsProject::updateAfterBuild()
 {
     QTC_ASSERT(m_qbsProject.isValid(), return);
-    m_projectData = m_qbsProject.projectData();
+    const qbs::ProjectData &projectData = m_qbsProject.projectData();
+    if (projectData == m_projectData)
+        return;
+    qCDebug(qbsPmLog) << "Updating data after build";
+    m_projectData = projectData;
+    rootProjectNode()->update();
     updateBuildTargetData();
     updateCppCompilerCallData();
     if (m_extraCompilersPending) {
@@ -750,7 +763,7 @@ void QbsProject::updateDocuments(const QSet<QString> &files)
     m_qbsDocuments.unite(toAdd);
 }
 
-static CppTools::ProjectFile::Kind cppFileType(const qbs::SourceArtifact &sourceFile)
+static CppTools::ProjectFile::Kind cppFileType(const qbs::ArtifactData &sourceFile)
 {
     if (sourceFile.fileTags().contains(QLatin1String("hpp")))
         return CppTools::ProjectFile::CXXHeader;
@@ -773,6 +786,89 @@ static QString groupLocationToProjectFile(const qbs::CodeLocation &location)
                         .arg(location.column());
 }
 
+// TODO: Receive the values from qbs when QBS-1030 is resolved.
+static void getExpandedCompilerFlags(QStringList &cFlags, QStringList &cxxFlags,
+                                     const qbs::PropertyMap &properties)
+{
+    const auto getCppProp = [properties](const char *propertyName) {
+        return properties.getModuleProperty("cpp", QLatin1String(propertyName));
+    };
+    const QVariant &enableExceptions = getCppProp("enableExceptions");
+    const QVariant &enableRtti = getCppProp("enableRtti");
+    QStringList commonFlags = getCppProp("platformCommonCompilerFlags").toStringList();
+    commonFlags << getCppProp("commonCompilerFlags").toStringList()
+                << getCppProp("platformDriverFlags").toStringList()
+                << getCppProp("driverFlags").toStringList();
+    const QStringList toolchain = properties.getModulePropertiesAsStringList("qbs", "toolchain");
+    if (toolchain.contains("gcc")) {
+        bool hasTargetOption = false;
+        if (toolchain.contains("clang")) {
+            const int majorVersion = getCppProp("compilerVersionMajor").toInt();
+            const int minorVersion = getCppProp("compilerVersionMinor").toInt();
+            if (majorVersion > 3 || (majorVersion == 3 && minorVersion >= 1))
+                hasTargetOption = true;
+        }
+        if (hasTargetOption) {
+            commonFlags << "-target" << getCppProp("target").toString();
+        } else {
+            const QString targetArch = getCppProp("targetArch").toString();
+            if (targetArch == "x86_64")
+                commonFlags << "-m64";
+            else if (targetArch == "i386")
+                commonFlags << "-m32";
+            const QString machineType = getCppProp("machineType").toString();
+            if (!machineType.isEmpty())
+                commonFlags << ("-march=" + machineType);
+        }
+        const QStringList targetOS = properties.getModulePropertiesAsStringList(
+                    "qbs", "targetOS");
+        if (targetOS.contains("unix")) {
+            const QVariant positionIndependentCode = getCppProp("positionIndependentCode");
+            if (!positionIndependentCode.isValid() || positionIndependentCode.toBool())
+                commonFlags << "-fPIC";
+        }
+        cFlags = cxxFlags = commonFlags;
+
+        const QString cxxLanguageVersion = getCppProp("cxxLanguageVersion").toString();
+        if (cxxLanguageVersion == "c++11")
+            cxxFlags << "-std=c++0x";
+        else if (cxxLanguageVersion == "c++14")
+            cxxFlags << "-std=c++1y";
+        else if (!cxxLanguageVersion.isEmpty())
+            cxxFlags << ("-std=" + cxxLanguageVersion);
+        const QString cxxStandardLibrary = getCppProp("cxxStandardLibrary").toString();
+        if (!cxxStandardLibrary.isEmpty() && toolchain.contains("clang"))
+            cxxFlags << ("-stdlib=" + cxxStandardLibrary);
+        if (enableExceptions.isValid()) {
+            cxxFlags << QLatin1String(enableExceptions.toBool()
+                                      ? "-fexceptions" : "-fno-exceptions");
+        }
+        if (enableRtti.isValid())
+            cxxFlags << QLatin1String(enableRtti.toBool() ? "-frtti" : "-fno-rtti");
+
+        const QString cLanguageVersion = getCppProp("cLanguageVersion").toString();
+        if (cLanguageVersion == "c11")
+            cFlags << "-std=c1x";
+        else if (!cLanguageVersion.isEmpty())
+            cFlags << ("-std=" + cLanguageVersion);
+    } else if (toolchain.contains("msvc")) {
+        if (enableExceptions.toBool()) {
+            const QString exceptionModel = getCppProp("exceptionHandlingModel").toString();
+            if (exceptionModel == "default")
+                commonFlags << "/EHsc";
+            else if (exceptionModel == "seh")
+                commonFlags << "/EHa";
+            else if (exceptionModel == "externc")
+                commonFlags << "/EHs";
+        }
+        cFlags = cxxFlags = commonFlags;
+        cFlags << "/TC";
+        cxxFlags << "/TP";
+        if (enableRtti.isValid())
+            cxxFlags << QLatin1String(enableRtti.toBool() ? "/GR" : "/GR-");
+    }
+}
+
 void QbsProject::updateCppCodeModel()
 {
     if (!m_projectData.isValid())
@@ -802,15 +898,35 @@ void QbsProject::updateCppCodeModel()
     qDeleteAll(m_extraCompilers);
     m_extraCompilers.clear();
     foreach (const qbs::ProductData &prd, m_projectData.allProducts()) {
+        QString cPch;
+        QString cxxPch;
+        QString objcPch;
+        QString objcxxPch;
+        const auto &pchFinder = [&cPch, &cxxPch, &objcPch, &objcxxPch](const qbs::ArtifactData &a) {
+            if (a.fileTags().contains("c_pch_src"))
+                cPch = a.filePath();
+            else if (a.fileTags().contains("cpp_pch_src"))
+                cxxPch = a.filePath();
+            else if (a.fileTags().contains("objc_pch_src"))
+                objcPch = a.filePath();
+            else if (a.fileTags().contains("objcpp_pch_src"))
+                objcxxPch = a.filePath();
+        };
+        const QList<qbs::ArtifactData> &generatedArtifacts = prd.generatedArtifacts();
+        std::for_each(generatedArtifacts.cbegin(), generatedArtifacts.cend(), pchFinder);
+        foreach (const qbs::GroupData &grp, prd.groups()) {
+            const QList<qbs::ArtifactData> &sourceArtifacts = grp.allSourceArtifacts();
+            std::for_each(sourceArtifacts.cbegin(), sourceArtifacts.cend(), pchFinder);
+        }
+
         foreach (const qbs::GroupData &grp, prd.groups()) {
             const qbs::PropertyMap &props = grp.properties();
 
-            ppBuilder.setCxxFlags(props.getModulePropertiesAsStringList(
-                                      QLatin1String(CONFIG_CPP_MODULE),
-                                      QLatin1String(CONFIG_CXXFLAGS)));
-            ppBuilder.setCFlags(props.getModulePropertiesAsStringList(
-                                    QLatin1String(CONFIG_CPP_MODULE),
-                                    QLatin1String(CONFIG_CFLAGS)));
+            QStringList cFlags;
+            QStringList cxxFlags;
+            getExpandedCompilerFlags(cFlags, cxxFlags, props);
+            ppBuilder.setCxxFlags(cxxFlags);
+            ppBuilder.setCFlags(cFlags);
 
             QStringList list = props.getModulePropertiesAsStringList(
                         QLatin1String(CONFIG_CPP_MODULE),
@@ -831,6 +947,7 @@ void QbsProject::updateCppCodeModel()
                                                          QLatin1String(CONFIG_INCLUDEPATHS));
             list.append(props.getModulePropertiesAsStringList(QLatin1String(CONFIG_CPP_MODULE),
                                                               QLatin1String(CONFIG_SYSTEM_INCLUDEPATHS)));
+            list.removeDuplicates();
             CppTools::ProjectPartHeaderPaths grpHeaderPaths;
             foreach (const QString &p, list)
                 grpHeaderPaths += CppTools::ProjectPartHeaderPath(
@@ -841,6 +958,7 @@ void QbsProject::updateCppCodeModel()
                                                          QLatin1String(CONFIG_FRAMEWORKPATHS));
             list.append(props.getModulePropertiesAsStringList(QLatin1String(CONFIG_CPP_MODULE),
                                                               QLatin1String(CONFIG_SYSTEM_FRAMEWORKPATHS)));
+            list.removeDuplicates();
             foreach (const QString &p, list)
                 grpHeaderPaths += CppTools::ProjectPartHeaderPath(
                             FileName::fromUserInput(p).toString(),
@@ -848,18 +966,26 @@ void QbsProject::updateCppCodeModel()
 
             ppBuilder.setHeaderPaths(grpHeaderPaths);
 
-            const QString pch = props.getModuleProperty(QLatin1String(CONFIG_CPP_MODULE),
-                    QLatin1String(CONFIG_PRECOMPILEDHEADER)).toString();
-            ppBuilder.setPreCompiledHeaders(QStringList() << pch);
-
             ppBuilder.setDisplayName(grp.name());
             ppBuilder.setProjectFile(groupLocationToProjectFile(grp.location()));
 
-            QHash<QString, qbs::SourceArtifact> filePathToSourceArtifact;
-            foreach (const qbs::SourceArtifact &source, grp.allSourceArtifacts()) {
+            QHash<QString, qbs::ArtifactData> filePathToSourceArtifact;
+            bool hasCFiles = false;
+            bool hasCxxFiles = false;
+            bool hasObjcFiles = false;
+            bool hasObjcxxFiles = false;
+            foreach (const qbs::ArtifactData &source, grp.allSourceArtifacts()) {
                 filePathToSourceArtifact.insert(source.filePath(), source);
 
                 foreach (const QString &tag, source.fileTags()) {
+                    if (tag == "c")
+                        hasCFiles = true;
+                    else if (tag == "cpp")
+                        hasCxxFiles = true;
+                    else if (tag == "objc")
+                        hasObjcFiles = true;
+                    else if (tag == "objcpp")
+                        hasObjcxxFiles = true;
                     for (auto i = factoriesBegin; i != factoriesEnd; ++i) {
                         if ((*i)->sourceTag() != tag)
                             continue;
@@ -881,6 +1007,31 @@ void QbsProject::updateCppCodeModel()
                 }
             }
 
+            QStringList pchFiles;
+            if (hasCFiles && props.getModuleProperty("cpp", "useCPrecompiledHeader").toBool()
+                    && !cPch.isEmpty()) {
+                pchFiles << cPch;
+            }
+            if (hasCxxFiles && props.getModuleProperty("cpp", "useCxxPrecompiledHeader").toBool()
+                    && !cxxPch.isEmpty()) {
+                pchFiles << cxxPch;
+            }
+            if (hasObjcFiles && props.getModuleProperty("cpp", "useObjcPrecompiledHeader").toBool()
+                    && !objcPch.isEmpty()) {
+                pchFiles << objcPch;
+            }
+            if (hasObjcxxFiles
+                    && props.getModuleProperty("cpp", "useObjcxxPrecompiledHeader").toBool()
+                    && !objcxxPch.isEmpty()) {
+                pchFiles << objcxxPch;
+            }
+            if (pchFiles.count() > 1) {
+                qCWarning(qbsPmLog) << "More than one pch file enabled for source files in group"
+                                    << grp.name() << "in product" << prd.name();
+                qCWarning(qbsPmLog) << "Expect problems with code model";
+            }
+            ppBuilder.setPreCompiledHeaders(pchFiles);
+
             const QList<Id> languages = ppBuilder.createProjectPartsForFiles(
                 grp.allFilePaths(),
                 [filePathToSourceArtifact](const QString &filePath) {
@@ -920,7 +1071,7 @@ void QbsProject::updateCppCompilerCallData()
             CppTools::ProjectInfo::CompilerCallGroup compilerCallGroup;
             compilerCallGroup.groupId = groupLocationToProjectFile(group.location());
 
-            foreach (const qbs::SourceArtifact &file, group.allSourceArtifacts()) {
+            foreach (const qbs::ArtifactData &file, group.allSourceArtifacts()) {
                 const QString &filePath = file.filePath();
                 if (!CppTools::ProjectFile::isSource(cppFileType(file)))
                     continue;
@@ -982,7 +1133,7 @@ void QbsProject::updateApplicationTargets()
                     FileName::fromString(productData.location().filePath()));
             continue;
         }
-        foreach (const qbs::TargetArtifact &ta, productData.targetArtifacts()) {
+        foreach (const qbs::ArtifactData &ta, productData.targetArtifacts()) {
             QTC_ASSERT(ta.isValid(), continue);
             if (!ta.isExecutable())
                 continue;
@@ -998,11 +1149,8 @@ void QbsProject::updateDeploymentInfo()
 {
     DeploymentData deploymentData;
     if (m_qbsProject.isValid()) {
-        qbs::InstallOptions installOptions;
-        installOptions.setInstallRoot(QLatin1String("/"));
-        foreach (const qbs::InstallableFile &f, m_qbsProject
-                     .installableFilesForProject(m_projectData, installOptions)) {
-            deploymentData.addFile(f.sourceFilePath(), QFileInfo(f.targetFilePath()).path(),
+        foreach (const qbs::ArtifactData &f, m_projectData.installableArtifacts()) {
+            deploymentData.addFile(f.filePath(), f.installData().installDir(),
                     f.isExecutable() ? DeployableFile::TypeExecutable : DeployableFile::TypeNormal);
         }
     }
